import BIT.highBIT.*;
import java.io.*;
import java.util.*;

// Written by Christian Collberg, collberg@cs.arizona.edu
public class LCount {
    private static Hashtable ht = new Hashtable();    
    private static String methodName;
    private static int totalInstr = 0;

    /* main reads in all the files class files present in the input directory,
     * instruments them, and outputs them to the specified output directory.
     */
    public static void main(String argv[]) {
        File file_in = new File(argv[0]);
        String infilenames[] = file_in.list();

        for (int i = 0; i < infilenames.length; i++) {
            String infilename = infilenames[i];
            if (infilename.endsWith(".class")) {
	       ClassInfo ci = new ClassInfo(argv[0] + System.getProperty("file.separator") + infilename);
			
                for (Enumeration e = ci.getRoutines().elements(); e.hasMoreElements(); ) {
                    Routine routine = (Routine) e.nextElement();
                    String className  = ci.getClassName();
                    String methodName = routine.getMethodName();
		    routine.addBefore("LCount", "setMethodName", className + " " + methodName);
                    HashSet seen = new HashSet();
	            for (Enumeration instrs = (routine.getInstructionArray()).elements(); instrs.hasMoreElements(); ) {
	               Instruction instr = (Instruction) instrs.nextElement();
                       int pc  = instr.getOffset();
                       int lineNr  = routine.getLineNumber(pc);
                       if (lineNr > 0) {
                          boolean notFound = seen.add(new Integer(lineNr));
                          if (notFound)
                             instr.addBefore("LCount", "count", lineNr+"");
                       }
		       int opcode=instr.getOpcode();
                       if ((opcode == InstructionTable.invokeinterface) ||
                           (opcode == InstructionTable.invokespecial) ||
                           (opcode == InstructionTable.invokestatic) ||
                           (opcode == InstructionTable.invokevirtual))
		          instr.addAfter("LCount", "setMethodName", className + " " + methodName);
		       }
                }
                ci.addAfter("LCount", "printLCount", ci.getClassName());
                ci.write(argv[1] + System.getProperty("file.separator") + infilename);
            }
        }
    }
    
    public static synchronized void setMethodName(String name) {
       methodName = name;
    }

    public static synchronized void printLCount(String foo) {
        Enumeration keys = ht.keys();
        while (keys.hasMoreElements()) {
            String word = (String)keys.nextElement();
            Integer count = (Integer) ht.get(word);
            int val = count.intValue();
            double p1 = ((double) val) / ((double)totalInstr) * 100.0; 
            double p2 = java.lang.Math.rint(p1*10.0) / 10.0; 
            String percent;
            if (p2 < 10.0)
               percent = "0" + p2;
            else
               percent = "" + p2;
            System.out.println(word + " " + val + " " + percent);           
        }
    }
    

    public static synchronized void count(String lineNr) {
        String s = methodName + " " + lineNr;
        Integer c = (Integer) ht.get(s);
        totalInstr++;
        if (c == null) 
           ht.put(s, new Integer(1));
        else 
           ht.put(s, new Integer(c.intValue()+1));
    }

}

