
#include "ast.h"

class IntStack
{
    public:
	void Push(int type) { info.Next() = type; }
	void Pop()          { if (info.Length() > 0) info.Reset(info.Length() - 1); }
	int Size()          { return info.Length(); }
	int Top()           { return (int) (info.Length() > 0 ? info[info.Length() - 1] : NULL); }
    private:
	Tuple<int> info;
};

struct jump
{
	int statement, loop;
	jump(int s, int l) { statement = s; loop = l; }
};

class JumpStack
{
    public:
	void Push(jump *type) { info.Next() = type; }
	void Pop()            { if (info.Length() > 0) info.Reset(info.Length() - 1); }
	int Size()            { return info.Length(); }
	jump *Top()           { return (jump*) (info.Length() > 0 ? info[info.Length() - 1] : NULL); }
    private:
	Tuple<jump *> info;
};

struct Throw
{
	int statement;
	TypeSymbol *type;
	Throw(int s, TypeSymbol *t) { statement = s; type = t; }
};

class ThrowStack
{
    public:
	void Push(Throw *type) { info.Next() = type; }
	void Pop()           { if (info.Length() > 0) info.Reset(info.Length() - 1); }
	int Size()           { return info.Length(); }
	Throw *Top()         { return (Throw*) (info.Length() > 0 ? info[info.Length() - 1] : NULL); }
	void Delete(int i) 
	{
		delete info[i];
		for (; i < info.Length()-1; i++)
			info[i] = info[i+1];
		Pop();
	}
    Throw *operator[](const int i)  { return info[i]; }
    private:
	Tuple<Throw *> info;
};

struct ThrowCatch
{
	int statement, clause;
	ThrowCatch(int s, int c) { statement = s; clause = c; }
};

class ThrowCatchStack
{
    public:
	void Push(ThrowCatch *type) { info.Next() = type; }
	void Pop()           { if (info.Length() > 0) info.Reset(info.Length() - 1); }
	int Size()           { return info.Length(); }
	ThrowCatch *Top()    { return (ThrowCatch*) (info.Length() > 0 ? info[info.Length() - 1] : NULL); }
    private:
	Tuple<ThrowCatch *> info;
};

class MP_AstStack
{
    public:
	void Push(MP_Ast *type) { info.Next() = type; }
	void Pop()           { if (info.Length() > 0) info.Reset(info.Length() - 1); }
	int Size()           { return info.Length(); }
	MP_Ast *Top()        { return (info.Length() > 0 ? info[info.Length() - 1] : NULL); }
    MP_Ast *operator[](const int i)  { return info[i]; }
    private:
	Tuple<MP_Ast *> info;
};

struct call
{
	int statement, subs;
	call(int statement_, int subs_) { statement = statement_; subs = subs_; }
};

class CallStack
{
    public:
	void Push(call *type) { info.Next() = type; }
	void Pop()            { if (info.Length() > 0) info.Reset(info.Length() - 1); }
	int Size()            { return info.Length(); }
	call *Top()           { return (call*) (info.Length() > 0 ? info[info.Length() - 1] : NULL); }
    private:
	Tuple<call *> info;
};
