#ifndef mp_ast_INCLUDED
#define mp_ast_INCLUDED

#include <iostream.h>
#include "unicode.h"

class MP_Ast
{
 public:
    static   bool     line_numbers;
	virtual  void     Print(ostream &out) {;}
	virtual  void     PrintText(ostream &out) {;}

 protected:
	         int      line_num;
	static   int      num_boxes;
             void     Box(ostream &out, char *box_type,
						  int box1, int box2, int box3, int box4);
			 void     BoxArg(ostream &out, char *box_type, const char *arg,
							 int box1, int box2, int box3, int box4);
			 void     BoxBoxMath(ostream &out, char *box_type, 
								 int box, wchar_t *contents);
             void     BoxDummy(ostream &out);
			 void     BoxMacro(ostream &out, char *macro_name, const wchar_t *arg, 
							   const wchar_t *arg2, const wchar_t *arg3);
			 void     BoxMacroText(ostream &out, char *macro_name, 
								   wchar_t *arg2, wchar_t *arg3);
             void     BoxMath(ostream &out, char *box_type, wchar_t *contents);
			 void     BoxMathBox(ostream &out, char *box_type, 
								 wchar_t *contents, int box);
			 void     BoxString(ostream &out, char *box_type, wchar_t *string);
             void     BoxTeX(ostream &out, char *box_type, wchar_t *s1,
							 wchar_t *s2, wchar_t *s3);
             void     BoxText(ostream &out, char *box_type);
			 void     LoopClose();
			 void     StartBox(ostream &out, char *box_type, bool TeX = false, bool math = false);
             void     StatSeq(ostream &out, char *seq_string, 
							  MP_Ast *stats[], int num_stats);
             void     TeXOut(ostream &out, const char *str);
             void     TeXOut(ostream &out, const wchar_t *str);
};

// --------------------------------------------------------------------------

class MP_AndThen : public MP_Ast
{
             MP_Ast   *left, *right;

 public:
			          MP_AndThen(int line_num, MP_Ast *left, MP_Ast *right)
					  {
						  this -> line_num = line_num;
						  this -> left = left;
						  this -> right = right;
					  }
	virtual           ~MP_AndThen() {delete left; delete right;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_ArrayAccess : public MP_Ast
{
             MP_Ast   *array, *expr;

 public:
			          MP_ArrayAccess(int line_num, MP_Ast *array, MP_Ast *expr)
					  {
						  this -> line_num = line_num;
						  this -> array = array;
						  this -> expr = expr;
					  }
	virtual           ~MP_ArrayAccess() {delete array; delete expr;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_ArrayInit : public MP_Ast
{
     	     char     *data;  // not wchar_t!!

 public:
			          MP_ArrayInit(int line_num, char *data)
					  {
						  this -> line_num = line_num;
						  this -> data = data;
					  }
	virtual	          ~MP_ArrayInit() {delete data;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_BinExpr : public MP_Ast
{
             MP_Ast   *left, *op, *right;

 public:
			          MP_BinExpr(int line_num, MP_Ast *left, MP_Ast *op, MP_Ast *right)
					  {
						  this -> line_num = line_num;
						  this -> left = left;
						  this -> op = op;
						  this -> right = right;
					  }
	virtual           ~MP_BinExpr() {delete left; delete op; delete right;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Break : public MP_Ast
{
 public:
	                  MP_Break(int line_num) {this -> line_num = line_num;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_CaseBranch : public MP_Ast
{
	         MP_Ast   *stat, *expr;
			 bool     has_break, prev_has_break;

 public:
                      MP_CaseBranch(int line_num, MP_Ast *expr, MP_Ast *stat, bool has_break)
					  {
						  this -> line_num = line_num;
						  this -> expr = expr;
						  this -> stat = stat;
						  this -> has_break = has_break;
					  }
	virtual           ~MP_CaseBranch() {delete expr; delete stat;}
	virtual  void     Print(ostream &out);

	friend class MP_Case;
};

// --------------------------------------------------------------------------

class MP_DefaultCaseBranch : public MP_Ast
{
	         MP_Ast   *stat;
			 bool     prev_has_break;

 public:
                      MP_DefaultCaseBranch(int line_num, MP_Ast *stat)
					  {
						  this -> line_num = line_num;
						  this -> stat = stat;
					  }
	virtual           ~MP_DefaultCaseBranch() {delete stat;}
	virtual  void     Print(ostream &out);

	friend class MP_Case;
};

// --------------------------------------------------------------------------

class MP_Case : public MP_Ast
{
	         MP_Ast   *expr;
             int      num_cases;
        MP_CaseBranch **cases;
 MP_DefaultCaseBranch *default_case;

 public:
			          MP_Case(int line_num,
							  MP_Ast *expr,
							  int num_cases,
							  MP_CaseBranch **cases, 
							  MP_DefaultCaseBranch *default_case)
                      {
						  this -> line_num = line_num;
						  this -> expr = expr;
						  this -> num_cases = num_cases;
						  this -> cases = new MP_CaseBranch*[num_cases];
						  for (int i=0; i<num_cases; i++)
							  this -> cases[i] = cases[i];
						  this -> default_case = default_case;
					  }
	virtual	          ~MP_Case()
                      {
						  delete expr;
						  for (int i=0; i<num_cases; i++)
							  delete cases[i];
						  delete [] cases;
						  if (default_case)
							  delete default_case;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Cast : public MP_Ast
{
	         MP_Ast   *type, *expr;

 public:
                      MP_Cast(int line_num, MP_Ast *type, MP_Ast *expr)
					  {
						  this -> line_num = line_num;
						  this -> type = type;
						  this -> expr = expr;
					  }
	virtual           ~MP_Cast() {delete type; delete expr;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Catch : public MP_Ast
{
	         MP_Ast   *expr, *stat;

 public:
                      MP_Catch(int line_num, MP_Ast *expr, MP_Ast *stat)
					  {
						  this -> line_num = line_num;
						  this -> expr = expr;
						  this -> stat = stat;
					  }
	virtual           ~MP_Catch() {delete expr; delete stat;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Conditional : public MP_Ast
{
             MP_Ast   *expr, *true_stat, *false_stat;

 public:
			          MP_Conditional(int line_num, MP_Ast *expr,
									 MP_Ast *true_stat, 
									 MP_Ast *false_stat)
                      {
						  this -> line_num = line_num;
						  this -> expr = expr;
						  this -> true_stat = true_stat;
						  this -> false_stat = false_stat;
					  }
	virtual	          ~MP_Conditional() 
                      {
						  delete expr; 
						  delete true_stat; 
						  delete false_stat;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Continue : public MP_Ast
{
 public:
	                  MP_Continue(int line_num) {this -> line_num = line_num;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_FalseLit : public MP_Ast
{
 public:
	                  MP_FalseLit(int line_num) {this -> line_num = line_num;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Field : public MP_Ast
{
     	     MP_Ast   *base;
			 wchar_t  *field;
			 wchar_t  *type;

 public:
			          MP_Field(int line_num, MP_Ast *base, wchar_t *field, wchar_t *type)
					  {
						  this -> line_num = line_num;
						  this -> base = base;
						  this -> field = field;
						  this -> type = type;
					  }
	virtual	          ~MP_Field() {delete base;}
	virtual  void     Print(ostream &out);
	virtual  void     PrintText(ostream &out);
};

// --------------------------------------------------------------------------

class MP_For : public MP_Ast
{
             int      i, num_inits, num_updates;
             MP_Ast   **inits, **updates;
	         MP_Ast   *condition, *stat;

 public:
			          MP_For(int line_num,
							 int num_inits,
							 MP_Ast **inits,
							 MP_Ast *condition, 
							 int num_updates,
							 MP_Ast **updates, 
							 MP_Ast *stat)
                      {
						  this -> line_num = line_num;
						  this -> num_inits = num_inits;
						  this -> inits = new MP_Ast*[num_inits];
						  for (i=0; i<num_inits; i++)
							  this -> inits[i] = inits[i];
						  this -> condition = condition;
						  this -> num_updates = num_updates;
						  this -> updates = new MP_Ast*[num_updates];
						  for (i=0; i<num_updates; i++)
							  this -> updates[i] = updates[i];
						  this -> stat = stat;
					  }
	virtual	          ~MP_For()
                      {
						  for (i=0; i<num_inits; i++)
							  delete inits[i];
						  delete [] inits;
						  delete condition;
						  for (i=0; i<num_updates; i++)
							  delete updates[i];
						  delete [] updates;
						  delete stat;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Formal : public MP_Ast
{
			 MP_Ast   *modifiers, *variable_id;

 public:
			          MP_Formal(int line_num,
								MP_Ast *modifiers, 
								MP_Ast *variable_id)
					  {
						  this -> line_num = line_num;
						  this -> modifiers = modifiers;
						  this -> variable_id = variable_id;
					  }
	virtual	          ~MP_Formal() {delete modifiers; delete variable_id;}
	virtual  void     Print(ostream &out);
	virtual  void     PrintText(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Formals : public MP_Ast
{
             int      num_formals;
             MP_Ast   **formals;

 public:
			          MP_Formals(int line_num, int num_formals, MP_Ast **formals)
					  {
						  this -> line_num = line_num;
						  this -> num_formals = num_formals;
						  this -> formals = new MP_Ast*[num_formals];
						  for (int i=0; i<num_formals; i++)
							  this -> formals[i] = formals[i];
					  }
	virtual           ~MP_Formals()
					  {
						  for (int i=0; i<num_formals; i++)
							  delete formals[i];
						  delete [] formals;
					  }
	virtual  void     Print(ostream &out);
	virtual  void     PrintText(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Identifier : public MP_Ast
{
	         wchar_t  *name;
			 wchar_t  *type;
			 wchar_t  *owner;

 public:
			          MP_Identifier(int line_num,
									wchar_t *name,
									wchar_t *type = NULL,
									wchar_t *owner = NULL)
                      {
						  this -> line_num = line_num;
						  this -> name = name;
						  this -> type = type;
						  this -> owner = owner;
					  }
	virtual  void     Print(ostream &out);
	virtual  void     PrintText(ostream &out) {Unicode::Out(out, name);}
};

// --------------------------------------------------------------------------

class MP_If : public MP_Ast
{
             int      num_ifs;
             MP_Ast   **exprs, **stats;
	         MP_Ast   *else_stat;

 public:
			          MP_If(int line_num,
							int num_ifs,
							MP_Ast **exprs,
							MP_Ast **stats,
							MP_Ast *else_stat)
                      {
						  this -> line_num = line_num;
						  this -> num_ifs = num_ifs;
						  this -> exprs = new MP_Ast*[num_ifs];
						  this -> stats = new MP_Ast*[num_ifs];
						  for (int i=0; i<num_ifs; i++)
						  {
							  this -> exprs[i] = exprs[i];
							  this -> stats[i] = stats[i];
						  }
						  this -> else_stat = else_stat;
					  }
	virtual	          ~MP_If()
                      {
						  for (int i=0; i<num_ifs; i++)
						  {
							  delete exprs[i];
							  delete stats[i];
						  }
						  delete [] exprs;
						  delete [] stats;
						  if (else_stat)
							  delete else_stat;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_IntLit : public MP_Ast
{
	         wchar_t  *value;

 public:
			          MP_IntLit(int line_num, wchar_t *value)
					  {
						  this -> line_num = line_num;
						  this -> value = value;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Modifiers : public MP_Ast
{
             int      num_modifiers;
             wchar_t  **modifiers;

 public:
			          MP_Modifiers(int line_num, int num_modifiers, wchar_t **modifiers)
					  {
						  this -> line_num = line_num;
						  this -> num_modifiers = num_modifiers;
						  this -> modifiers = new wchar_t*[num_modifiers];
						  for (int i=0; i<num_modifiers; i++)
							  this -> modifiers[i] = modifiers[i];
					  }
	virtual           ~MP_Modifiers()
					  {
						  delete [] modifiers;
					  }
	virtual  void     Print(ostream &out);
	virtual  void     PrintText(ostream &out);
};

// --------------------------------------------------------------------------

class MP_New : public MP_Ast
{
	         MP_Ast   *type;
             int      num_args;
			 MP_Ast   **args;

 public:
			          MP_New(int line_num,
							 MP_Ast *type,
							 int num_args,
							 MP_Ast **args) 
                      {
						  this -> line_num = line_num;
						  this -> type = type;
						  this -> num_args = num_args;
						  this -> args = new MP_Ast*[num_args];
						  for (int i=0; i<num_args; i++)
							  this -> args[i] = args[i];
					  }
	virtual	          ~MP_New()
                      {
						  delete type;
						  for (int i=0; i<num_args; i++)
							  delete args[i];
						  delete [] args;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_NullLit : public MP_Ast
{
 public:
	                  MP_NullLit(int line_num) {this -> line_num = line_num;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Operator : public MP_Ast
{
             wchar_t  *op;

 public:
			          MP_Operator(int line_num, wchar_t *op)
					  {
						  this -> line_num = line_num;
						  this -> op = op;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_OrThen : public MP_Ast
{
             MP_Ast   *left, *right;

 public:
			          MP_OrThen(int line_num, MP_Ast *left, MP_Ast *right)
					  {
						  this -> line_num = line_num;
						  this -> left = left;
						  this -> right = right;
					  }
	virtual           ~MP_OrThen() {delete left; delete right;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Parens : public MP_Ast
{
	         MP_Ast   *expr;

 public:
                      MP_Parens(int line_num, MP_Ast *expr)
					  {
						  this -> line_num = line_num;
						  this -> expr = expr;
					  }
	virtual           ~MP_Parens() {delete expr;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_PostUnary : public MP_Ast
{
	         MP_Ast   *expr;
			 wchar_t  *op;

 public:
                      MP_PostUnary(int line_num, MP_Ast *expr, wchar_t *op)
                      {
						  this -> line_num = line_num;
						  this -> expr = expr;
						  this -> op = op;
					  }
	virtual           ~MP_PostUnary() {delete expr;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_PreUnary : public MP_Ast
{
			 wchar_t  *op;
	         MP_Ast   *expr;

 public:
                      MP_PreUnary(int line_num, wchar_t *op, MP_Ast *expr)
                      {
						  this -> line_num = line_num;
						  this -> op = op;
						  this -> expr = expr;
					  }
	virtual           ~MP_PreUnary() {delete expr;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_ProcCall : public MP_Ast
{
	         MP_Ast   *proc;
             int      num_args;
			 MP_Ast   **args;
			 wchar_t  **argtypes;
			 wchar_t  *return_type;
             MP_Ast   *throws;
			 int      num_subs;
			 wchar_t  **subs;

 public:
			          MP_ProcCall(int line_num,
								  MP_Ast *proc,
								  int num_args,
								  MP_Ast **args, 
								  wchar_t **argtypes,
								  wchar_t *return_type,
								  MP_Ast *throws,
								  int num_subs = 1,
								  wchar_t **subs = NULL)
                      {
						  this -> line_num = line_num;
						  this -> proc = proc;
						  this -> num_args = num_args;
						  this -> args = new MP_Ast*[num_args];
						  for (int i=0; i<num_args; i++)
							  this -> args[i] = args[i];
						  this -> argtypes = new wchar_t*[num_args];
						  for (int i=0; i<num_args; i++)
							  this -> argtypes[i] = argtypes[i];
						  this -> return_type = return_type;
						  this -> throws = throws;
						  this -> num_subs = num_subs;
						  this -> subs = new wchar_t*[num_subs];
						  for (int i=0; i<num_subs; i++)
							  this -> subs[i] = subs[i];
					  }
	virtual	          ~MP_ProcCall()
                      {
						  delete proc;
						  for (int i=0; i<num_args; i++)
							  delete args[i];
						  delete [] args;
						  delete [] argtypes;
						  delete throws;
						  delete [] subs;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Procedure : public MP_Ast
{
	         wchar_t  *name;
			 MP_Ast   *formals;
			 MP_Ast   *modifiers;
			 MP_Ast   *type;
	         MP_Ast   *stat;

 public:
			          MP_Procedure(int line_num,
								   wchar_t *name, 
								   MP_Ast *formals,
								   MP_Ast *modifiers, 
								   MP_Ast *type, 
								   MP_Ast *stat)
                      {
						  this -> line_num = line_num;
						  this -> name = name;
						  this -> formals = formals;
						  this -> modifiers = modifiers;
						  this -> type = type;
						  this -> stat = stat;
					  }
			          ~MP_Procedure()
                      {
						  delete formals;
						  delete modifiers;
						  delete type; 
						  delete stat;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_RealLit : public MP_Ast
{
	         wchar_t  *value;

 public:
			          MP_RealLit(int line_num, wchar_t *value)
					  {
						  this -> line_num = line_num;
						  this -> value = value;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Repeat : public MP_Ast
{
	         MP_Ast   *expr, *stat;

 public:
			          MP_Repeat(int line_num, MP_Ast *stat, MP_Ast *expr)
					  {
						  this -> line_num = line_num;
						  this -> expr = expr;
						  this -> stat = stat;
					  }
	virtual           ~MP_Repeat() {delete expr; delete stat;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Return : public MP_Ast
{
	         MP_Ast   *expr;
 public:
                      MP_Return(int line_num, MP_Ast *expr)
					  {
						  this -> line_num = line_num;
						  this -> expr = expr;
					  }
	virtual           ~MP_Return() {if (expr) delete expr;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_StatSeq : public MP_Ast
{
             int      num_stats;
             MP_Ast   **stats;

 public:
			          MP_StatSeq(int line_num, int num_stats, MP_Ast **stats)
					  {
						  this -> line_num = line_num;
						  this -> num_stats = num_stats;
						  this -> stats = new MP_Ast*[num_stats];
						  for (int i=0; i<num_stats; i++)
							  this -> stats[i] = stats[i];
					  }
	virtual           ~MP_StatSeq()
					  {
						  for (int i=0; i<num_stats; i++)
							  delete stats[i];
						  delete [] stats;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_StringLit : public MP_Ast
{
	         wchar_t  *value;

 public:
			          MP_StringLit(int line_num, wchar_t *value)
					  {
						  this -> line_num = line_num;
						  this -> value = value;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Throw : public MP_Ast
{
	         MP_Ast   *expr;

 public:
                      MP_Throw(int line_num, MP_Ast *expr)
					  {
						  this -> line_num = line_num;
						  this -> expr = expr;
					  }
	virtual           ~MP_Throw() {delete expr;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Throws : public MP_Ast
{
             int      num_throws;
             wchar_t  **throws;

 public:
			          MP_Throws(int line_num, int num_throws, wchar_t **throws)
					  {
						  this -> line_num = line_num;
						  this -> num_throws = num_throws;
						  this -> throws = new wchar_t*[num_throws];
						  for (int i=0; i<num_throws; i++)
							  this -> throws[i] = throws[i];
					  }
	virtual           ~MP_Throws()
					  {
						  delete [] throws;
					  }
	virtual  void     Print(ostream &out);
	virtual  void     PrintText(ostream &out);
};

// --------------------------------------------------------------------------

class MP_TrueLit : public MP_Ast
{
 public:
	                  MP_TrueLit(int line_num) {this -> line_num = line_num;}
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Try : public MP_Ast
{
	         MP_Ast   *stat;
             int      num_catches;
             MP_Catch **catches;
             MP_Ast   *finally;

 public:
			          MP_Try(int line_num,
							 MP_Ast *stat,
							 int num_catches,
							 MP_Catch **catches, 
							 MP_Ast *finally)
                      {
						  this -> line_num = line_num;
						  this -> stat = stat;
						  this -> num_catches = num_catches;
						  this -> catches = new MP_Catch*[num_catches];
						  for (int i=0; i<num_catches; i++)
							  this -> catches[i] = catches[i];
						  this -> finally = finally;
					  }
	virtual	          ~MP_Try()
                      {
						  delete stat;
						  for (int i=0; i<num_catches; i++)
							  delete catches[i];
						  delete [] catches;
						  if (finally)
							  delete finally;
					  }
	virtual  void     Print(ostream &out);
};

// --------------------------------------------------------------------------

class MP_Type : public MP_Ast
{
 public:
	         wchar_t  *type;
             int      num_dimens;

			          MP_Type(int line_num, wchar_t *type, int num_dimens = 0)
                      {
						  this -> line_num = line_num;
						  this -> type = type;
						  this -> num_dimens = num_dimens;
					  }
	virtual  void     Print(ostream &out);
	virtual  void     PrintText(ostream &out);
};

// --------------------------------------------------------------------------

class MP_VariableDeclaration : public MP_Ast
{
	         MP_Ast   *type;
			 int      num_decls;
			 MP_Ast   **decls;

 public:
                      MP_VariableDeclaration(int line_num,
											 MP_Ast *type, 
											 int num_decls,
											 MP_Ast **decls)
                      {
						  this -> line_num = line_num;
						  this -> type = type;
						  this -> num_decls = num_decls;
						  this -> decls = new MP_Ast*[num_decls];
						  for (int i=0; i<num_decls; i++)
							  this -> decls[i] = decls[i];
					  }
	virtual           ~MP_VariableDeclaration() 
                      {
						  delete type;
						  for (int i=0; i<num_decls; i++)
							  delete decls[i];
						  delete [] decls;
					  }
	virtual  void     Print(ostream &out);
	virtual  void     PrintText(ostream &out);
};

// --------------------------------------------------------------------------

class MP_VariableDeclarationId : public MP_Ast
{
 public:
	         wchar_t  *name;
             int      num_dimens;

			          MP_VariableDeclarationId(int line_num, 
											   wchar_t *name,
											   int num_dimens = 0)
                      {
						  this -> line_num = line_num;
						  this -> name = name;
						  this -> num_dimens = num_dimens;
					  }
	virtual  void     Print(ostream &out);
	virtual  void     PrintText(ostream &out);
};

// --------------------------------------------------------------------------

class MP_While : public MP_Ast
{
	         MP_Ast   *expr, *stat;

 public:
			          MP_While(int line_num, MP_Ast *expr, MP_Ast *stat)
					  {
						  this -> line_num = line_num;
						  this -> expr = expr;
						  this -> stat = stat;
					  }
	virtual           ~MP_While() {delete expr; delete stat;}
	virtual  void     Print(ostream &out);
};

#endif
