
#include "ast.h"

bool Ast::verbatim = false;

static bool always_brace_blocks = true;
static bool else_if = false;
static bool first_non_block;
static int indent = 0;
static bool interface;
static int method_num;
static int num_fields;
static void Indent(ostream& out)
{
	for (int i=0; i<indent*4; i++)
		out << " ";
}
static bool BlockOpen(ostream& out, Ast *ast, bool allow_return)
{
	bool empty;

	if (ast -> kind == Ast::EMPTY_STATEMENT)
		empty = true;
	else if (!allow_return && ast -> kind == Ast::RETURN && !((AstReturnStatement*)ast) -> expression_opt)
		empty = true;
	else if (ast -> kind != Ast::BLOCK)
		empty = false;
	else if (ast -> BlockCast() -> NumStatements() <= 1)
		empty = ast -> BlockCast() -> NumStatements() == 0;
	else
		return BlockOpen(out, ast -> BlockCast() -> Statement(0), allow_return);

	out << "\n";
	Indent(out);
	out << "{\n";
	indent++;
	first_non_block = false;
	return !empty;
}
static void BlockClose(ostream& out, bool break_line = true)
{
	indent--;
	Indent(out);
	out << "}";
	if (break_line)
		out << "\n";
}
static void MethodHeader(ostream& out)
{
	if (method_num > 0 || num_fields > 0)
	{
		out << "\n";
		Indent(out);
		out << "// ---------------------------------------------\n";
	}
	method_num++;
}

void Ast::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    out << "Node number " << (int) kind << " does not contain a pretty print routine\n";
}

void AstBlock::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	int num_statements = NumStatements();

	//    Unicode::Out(lex_stream.Name(left_brace_token));
    Unicode::Out(out, label_token_opt ? lex_stream.Name(label_token_opt) : L"");

    if (num_statements > 0)
    {
		bool indent_block = nesting_level > 0 && 
			//Statement(0) -> kind != BLOCK && 
			//			Statement(num_statements - 1) -> kind != BLOCK &&
			//			(Statement(0) -> kind != FOR || num_statements > 1) && 
			first_non_block && 
			!else_if;

		if (indent_block)
		{
			if ((always_brace_blocks && *lex_stream.Name(left_brace_token) == '{') || num_statements > 1)
			{
				indent--;
				Indent(out);
				indent++;
				out << "{\n";
			}
		}
		for (int k = 0; k < num_statements; k++)
		{
			int kind = this -> Statement(k) -> kind;
			if (kind != BLOCK && kind != LOCAL_VARIABLE_DECLARATION)
				first_non_block = true;
			if (kind != RETURN && kind != BLOCK && !else_if)
				Indent(out);
			else_if = false;
			this -> Statement(k) -> PrettyPrint(out, lex_stream);
			if (kind != IF && kind != FOR && kind != BLOCK && kind != RETURN &&
				kind != TRY && kind != WHILE && kind != DO && kind != SWITCH &&
				kind != SYNCHRONIZED_STATEMENT && kind != SWITCH_BLOCK)
				out << ";\n";
		}
		if (indent_block)
		{
			if ((always_brace_blocks && *lex_stream.Name(left_brace_token) == '{') || num_statements > 1)
			{
				indent--;
				Indent(out);
				indent++;
				out << "}\n";
			}
		}
    }
	else if (nesting_level > 0)
	{
		Indent(out);
		out << ";\n";
	}
}

void AstPrimitiveType::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(primitive_kind_token));
}

void AstArrayType::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    type -> PrettyPrint(out, lex_stream);
    for (int i = 0; i < this -> NumBrackets(); i++)
		out << "[]";
}

void AstSimpleName::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(identifier_token));
}

void AstPackageDeclaration::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(package_token));
	out << " ";
    name -> PrettyPrint(out, lex_stream);
	out << ";\n";
}

void AstImportDeclaration::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(import_token));
    out << " ";
    name -> PrettyPrint(out, lex_stream);
	if (star_token_opt)
	{
		out << ".";
        Unicode::Out(out, lex_stream.Name(star_token_opt));
	}
    out << ";\n";
}

void AstCompilationUnit::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    if (package_declaration_opt)
		package_declaration_opt -> PrettyPrint(out, lex_stream);
    for (int m = 0; m < this -> NumImportDeclarations(); m++)
		this -> ImportDeclaration(m) -> PrettyPrint(out, lex_stream);
    for (int n = 0; n < this -> NumTypeDeclarations(); n++)
		this -> TypeDeclaration(n) -> PrettyPrint(out, lex_stream);
}

void AstModifier::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(modifier_kind_token));
}

void AstEmptyDeclaration::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(semicolon_token));
}

void AstClassBody::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    for (int k = 0; k < this -> NumClassBodyDeclarations(); k++)
		this -> ClassBodyDeclaration(k) -> PrettyPrint(out, lex_stream);
}

void AstClassDeclaration::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    for (int i = 0; i < this -> NumClassModifiers(); i++)
    {
		this -> ClassModifier(i) -> PrettyPrint(out, lex_stream);
		out << " ";
    }
    Unicode::Out(out, lex_stream.Name(class_token));
    out << " ";
    Unicode::Out(out, lex_stream.Name(identifier_token));
    out << " ";
    if (super_opt)
	{
		out << "extends ";
		super_opt -> PrettyPrint(out, lex_stream);
	}
    if (NumInterfaces() > 0)
    {
		if (super_opt)
			out << " ";
		out << "implements ";
		for (int k = 0; k < NumInterfaces(); k++)
		{
			this -> Interface(k) -> PrettyPrint(out, lex_stream);
			if (k < NumInterfaces() - 1)
				out << ", ";
		}
    }
	interface = false;
	method_num = 0;
	num_fields = 0;
    out << "\n{\n";
	indent++;
    class_body -> PrettyPrint(out, lex_stream);
	indent--;
    out << "}\n";
}

void AstArrayInitializer::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	out << (verbatim ? "{" : "\\{");
    for (int i = 0; i < NumVariableInitializers(); i++)
    {
		this -> VariableInitializer(i) -> PrettyPrint(out, lex_stream);
		if (i < NumVariableInitializers() - 1)
			out << ", ";
	}
	out << (verbatim ? "}" : "\\}");
}

void AstBrackets::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    out << "[]";
}

void AstVariableDeclaratorId::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	if (verbatim)
		Unicode::Out(out, lex_stream.Name(identifier_token));
	else
		Unicode::TeXOut(out, lex_stream.Name(identifier_token));
    for (int i = 0; i < NumBrackets(); i++)
		out << " []";
}

void AstVariableDeclarator::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    variable_declarator_name -> PrettyPrint(out, lex_stream);
    if (variable_initializer_opt)
    {
		out << " = ";
		variable_initializer_opt -> PrettyPrint(out, lex_stream);
    }
}

void AstFieldDeclaration::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	Indent(out);
    for (int i = 0; i < this -> NumVariableModifiers(); i++)
    {
		this -> VariableModifier(i) -> PrettyPrint(out, lex_stream);
		out << " ";
    }
    type -> PrettyPrint(out, lex_stream);
    out << " ";
    for (int k = 0; k < this -> NumVariableDeclarators(); k++)
    {
		this -> VariableDeclarator(k) -> PrettyPrint(out, lex_stream);
		if (k < NumVariableDeclarators() - 1)
			out << ", ";
    }
    out << ";\n";
	num_fields++;
}

void AstFormalParameter::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    for (int i = 0; i < this -> NumParameterModifiers(); i++)
    {
		this -> ParameterModifier(i) -> PrettyPrint(out, lex_stream);
		out << " ";
    }
    type -> PrettyPrint(out, lex_stream);
	out << " ";
    variable_declarator_name -> PrettyPrint(out, lex_stream);
}

void AstMethodDeclarator::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(identifier_token));
    out << "(";
    for (int j = 0; j < this -> NumFormalParameters(); j++)
    {
		this -> FormalParameter(j) -> PrettyPrint(out, lex_stream);
		if (j < this -> NumFormalParameters() - 1)
			out << ", ";
    }
    out << ")";
    for (int i = 0; i < NumBrackets(); i++)
		out << " []";
}

void AstMethodDeclaration::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	if (!interface)
		MethodHeader(out);
	Indent(out);
    for (int i = 0; i < this -> NumMethodModifiers(); i++)
    {
		this -> MethodModifier(i) -> PrettyPrint(out, lex_stream);
		out << " ";
    }
    type -> PrettyPrint(out, lex_stream);
	out << " ";
    method_declarator -> PrettyPrint(out, lex_stream);
    for (int k = 0; k < this -> NumThrows(); k++)
	{
		this -> Throw(k) -> PrettyPrint(out, lex_stream);
		if (k < this -> NumThrows() - 1)
			out << ", ";
	}
	if (interface)
		out << ";\n";
	else
	{
		if (BlockOpen(out, method_body, false))
			method_body -> PrettyPrint(out, lex_stream);
		BlockClose(out);
	}
}

void AstStaticInitializer::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	if (!verbatim)
		Unicode::Out(out, lex_stream.Name(static_token));
    block -> PrettyPrint(out, lex_stream);
}

void AstThisCall::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(this_token));
    out << "(";
    for (int j = 0; j < NumArguments(); j++)
	{
		this -> Argument(j) -> PrettyPrint(out, lex_stream);
		if (j < NumArguments() - 1)
			out << ", ";
	}
    out << ")";
}

void AstSuperCall::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	if (*lex_stream.Name(super_token) != 123)
	{
		Indent(out);
		if (base_opt)
			base_opt -> PrettyPrint(out, lex_stream);
		Unicode::Out(out, lex_stream.Name(super_token));
		if (NumArguments() > 0)
		{
			out << "(";
			for (int j = 0; j < NumArguments(); j++)
			{
				this -> Argument(j) -> PrettyPrint(out, lex_stream);
				if (j < NumArguments() - 1)
					out << ", ";
			}
			out << ");\n";
		}
	}
}

void AstConstructorBlock::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    if (explicit_constructor_invocation_opt)
	{
		if (explicit_constructor_invocation_opt -> kind != SUPER_CALL)
			Indent(out);
		explicit_constructor_invocation_opt -> PrettyPrint(out, lex_stream);
		if (explicit_constructor_invocation_opt -> kind != SUPER_CALL)
			out << ";\n";
	}
    block -> PrettyPrint(out, lex_stream);
}

void AstConstructorDeclaration::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	MethodHeader(out);
	Indent(out);
    for (int i = 0; i < this -> NumConstructorModifiers(); i++)
    {
		this -> ConstructorModifier(i) -> PrettyPrint(out, lex_stream);
		out << " ";
    }
    constructor_declarator -> PrettyPrint(out, lex_stream);
    if (this -> NumThrows() > 0)
    {
		out << " throws ";
		for (int k = 0; k < this -> NumThrows(); k++)
		{
			this -> Throw(k) -> PrettyPrint(out, lex_stream);
			if (k < this -> NumThrows() - 1)
				out << ", ";
		}
    }
	if (BlockOpen(out, constructor_body, false))
		constructor_body -> PrettyPrint(out, lex_stream);
	BlockClose(out);
}

void AstInterfaceDeclaration::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    for (int i = 0; i < this -> NumInterfaceModifiers(); i++)
    {
		this -> InterfaceModifier(i) -> PrettyPrint(out, lex_stream);
		out << " ";
    }
    Unicode::Out(out, lex_stream.Name(interface_token));
    out << " ";
    Unicode::Out(out, lex_stream.Name(identifier_token));
	if (NumExtendsInterfaces() > 0)
	{
		out << " extends ";
		for (int k = 0; k < NumExtendsInterfaces(); k++)
		{
			this -> ExtendsInterface(k) -> PrettyPrint(out, lex_stream);
			if (k < NumExtendsInterfaces() - 1)
				out << ", ";
		}
	}
    out << "\n";
	Indent(out);
	out << "{\n";
	interface = true;
	indent++;
    for (int n = 0; n < NumInterfaceMemberDeclarations(); n++)
		this -> InterfaceMemberDeclaration(n) -> PrettyPrint(out, lex_stream);
	indent--;
	Indent(out);
	out << "}\n";
}

void AstLocalVariableDeclarationStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    type -> PrettyPrint(out, lex_stream);
	out << " ";
    for (int k = 0; k < this -> NumVariableDeclarators(); k++)
    {
		this -> VariableDeclarator(k) -> PrettyPrint(out, lex_stream);
		if (k < this -> NumVariableDeclarators() - 1)
			out << ", ";
    }
}

void AstIfStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(if_token));
    out << " (";
    expression -> PrettyPrint(out, lex_stream);
    out << ")\n";
	indent++;
    true_statement -> PrettyPrint(out, lex_stream);
	indent--;
    if (false_statement_opt)
    {
		Indent(out);
		out << "else";
		if (false_statement_opt -> kind == IF ||
			(false_statement_opt -> kind == BLOCK && 
			 false_statement_opt -> BlockCast() -> NumStatements() == 1 && 
			 false_statement_opt -> BlockCast() -> Statement(0) -> kind == IF))
		{
			else_if = true;
			out << " ";
			false_statement_opt -> PrettyPrint(out, lex_stream);
		}
		else
		{
			out << "\n";
			indent++;
			false_statement_opt -> PrettyPrint(out, lex_stream);
			indent--;
		}
    }
}

void AstEmptyStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(semicolon_token));
}

void AstExpressionStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    expression -> PrettyPrint(out, lex_stream);
}

void AstCaseLabel::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(case_token));
    out << " ";
    expression -> PrettyPrint(out, lex_stream);
	out << ":";
}

void AstDefaultLabel::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(default_token));
    out << ":";
}

void AstSwitchBlockStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    for (int j = 0; j < NumSwitchLabels(); j++)
	{
		this -> SwitchLabel(j) -> PrettyPrint(out, lex_stream);
		if (j < NumSwitchLabels() - 1)
		{
			out << "\n";
			Indent(out);
		}
	}
	out << "\n";
	indent++;
    for (int l = 0; l < NumStatements(); l++)
	{
		int kind = this -> Statement(l) -> kind;
		if (kind != BLOCK && kind != RETURN)
			Indent(out);
		this -> Statement(l) -> PrettyPrint(out, lex_stream);
		if (kind != IF && kind != FOR && kind != BLOCK && kind != RETURN &&
			kind != TRY && kind != WHILE && kind != DO && kind != SWITCH &&
			kind != SYNCHRONIZED_STATEMENT)
			out << ";\n";
	}
	indent--;
}

void AstSwitchStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(switch_token));
    out << " (";
    expression -> PrettyPrint(out, lex_stream);
	out << ")";
	if (BlockOpen(out, switch_block, false))
		switch_block -> PrettyPrint(out, lex_stream);
	BlockClose(out, false);
	out << ";\n";
}

void AstWhileStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(while_token));
    out << " (";
    expression -> PrettyPrint(out, lex_stream);
	out << ")\n";
	indent++;
    statement -> PrettyPrint(out, lex_stream);
	indent--;
}

void AstDoStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(do_token));
	indent++;
    statement -> PrettyPrint(out, lex_stream);
	indent--;
	Indent(out);
    Unicode::Out(out, lex_stream.Name(while_token));
    out << " (";
    expression -> PrettyPrint(out, lex_stream);
	out << ")\n";
}

void AstForStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(for_token));
    out << " (";
    for (int m = 0; m < this -> NumForInitStatements(); m++)
    {
		this -> ForInitStatement(m) -> PrettyPrint(out, lex_stream);
		if (m < this -> NumForInitStatements() - 1)
			out << ", ";
    }
    out << "; ";
    if (end_expression_opt)
		end_expression_opt -> PrettyPrint(out, lex_stream);
    out << "; ";
    for (int n = 0; n < this -> NumForUpdateStatements(); n++)
    {
		this -> ForUpdateStatement(n) -> PrettyPrint(out, lex_stream);
		if (n < this -> NumForUpdateStatements() - 1)
			out << ", ";
    }
    out << ")\n";
	indent++;
    statement -> PrettyPrint(out, lex_stream);
	indent--;
}

void AstBreakStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(break_token));
	if (identifier_token_opt)
	{
		out << " ";
		Unicode::Out(out, lex_stream.Name(identifier_token_opt));
	}
}

void AstContinueStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(continue_token));
	if (identifier_token_opt)
	{
		out << " ";
		Unicode::Out(out, lex_stream.Name(identifier_token_opt));
	}
}

void AstReturnStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	if (!generated)
	{
		Indent(out);
		Unicode::Out(out, lex_stream.Name(return_token));
		if (expression_opt)
		{
			out << " ";
			expression_opt -> PrettyPrint(out, lex_stream);
		}
		out << ";\n";
	}
}

void AstThrowStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(throw_token));
    out << " ";
    expression -> PrettyPrint(out, lex_stream);
}

void AstSynchronizedStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(synchronized_token));
    out << " (";
    expression -> PrettyPrint(out, lex_stream);
	out << ")";
	if (BlockOpen(out, block, true))
		block -> PrettyPrint(out, lex_stream);
	BlockClose(out, false);
	out << ";\n";
}

void AstCatchClause::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	Indent(out);
    Unicode::Out(out, lex_stream.Name(catch_token));
	out << " (";
    formal_parameter -> PrettyPrint(out, lex_stream);
	out << ")";
	if (BlockOpen(out, block, true))
		block -> PrettyPrint(out, lex_stream);
	BlockClose(out);
}

void AstFinallyClause::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	Indent(out);
    Unicode::Out(out, lex_stream.Name(finally_token));
	if (BlockOpen(out, block, true))
		block -> PrettyPrint(out, lex_stream);
	BlockClose(out);
}

void AstTryStatement::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(try_token));
	if (BlockOpen(out, block, true))
		block -> PrettyPrint(out, lex_stream);
	BlockClose(out);
    for (int k = 0; k < this -> NumCatchClauses(); k++)
		this -> CatchClause(k) -> PrettyPrint(out, lex_stream);
    if (finally_clause_opt)
		finally_clause_opt -> PrettyPrint(out, lex_stream);
}

void AstIntegerLiteral::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(integer_literal_token));
}

void AstLongLiteral::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(long_literal_token));
}

void AstFloatingPointLiteral::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(floating_point_literal_token));
}

void AstDoubleLiteral::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(double_literal_token));
}

void AstTrueLiteral::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(true_literal_token));
}

void AstFalseLiteral::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(false_literal_token));
}

void AstStringLiteral::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	if (verbatim)
		Unicode::Out(out, lex_stream.Name(string_literal_token));
	else
		Unicode::TeXOut(out, lex_stream.Name(string_literal_token));
}

void AstCharacterLiteral::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	if (verbatim)
		Unicode::Out(out, lex_stream.Name(character_literal_token));
	else
		Unicode::TeXOut(out, lex_stream.Name(character_literal_token));
}

void AstNullLiteral::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(null_token));
}

void AstThisExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(this_token));
}

void AstSuperExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(super_token));
}

void AstParenthesizedExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(left_parenthesis_token));
    expression -> PrettyPrint(out, lex_stream);
    Unicode::Out(out, lex_stream.Name(right_parenthesis_token));
}

void AstTypeExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    type -> PrettyPrint(out, lex_stream);
}

void AstClassInstanceCreationExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(new_token));
	out << " ";
    if (dot_token_opt /* base_opt - see ast.h for explanation */)
		base_opt -> PrettyPrint(out, lex_stream);
    class_type -> PrettyPrint(out, lex_stream);
    out << "(";
    for (int j = 0; j < NumArguments(); j++)
	{
		this -> Argument(j) -> PrettyPrint(out, lex_stream);
		if (j < NumArguments() - 1)
			out << ", ";
	}
    out << ")";
    if (class_body_opt)
    {
		int save_method_num = method_num;
		method_num = 0;
		if (BlockOpen(out, class_body_opt, true))
			class_body_opt -> PrettyPrint(out, lex_stream);
		BlockClose(out, false);
		method_num = save_method_num;
    }
}

void AstDimExpr::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    out << "[";
    expression -> PrettyPrint(out, lex_stream);
	out << "]";
}

void AstArrayCreationExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(new_token));
	out << " ";
    array_type -> PrettyPrint(out, lex_stream);
    for (int k = 0; k < NumBrackets(); k++)
		out << " []";
    for (int j = 0; j < this -> NumDimExprs(); j++)
		DimExpr(j) -> PrettyPrint(out, lex_stream);
}

void AstFieldAccess::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    base -> PrettyPrint(out, lex_stream);
    out << ".";
	if (verbatim)
		Unicode::Out(out, lex_stream.Name(identifier_token));
	else
		Unicode::TeXOut(out, lex_stream.Name(identifier_token));
}

void AstMethodInvocation::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    method -> PrettyPrint(out, lex_stream);
    out << "(";
    for (int j = 0; j < NumArguments(); j++)
	{
		this -> Argument(j) -> PrettyPrint(out, lex_stream);
		if (j < NumArguments() - 1)
			out << ", ";
	}
    out << ")";
}

void AstArrayAccess::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    base -> PrettyPrint(out, lex_stream);
	out << "[";
    expression -> PrettyPrint(out, lex_stream);
	out << "]";
}

void AstPostUnaryExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    expression -> PrettyPrint(out, lex_stream);
    Unicode::Out(out, lex_stream.Name(post_operator_token));
}

void AstPreUnaryExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    Unicode::Out(out, lex_stream.Name(pre_operator_token));
    expression -> PrettyPrint(out, lex_stream);
}

void AstCastExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
	if (type_opt)
	{
		out << "(";
		type_opt -> PrettyPrint(out, lex_stream);
		out << ")";
	}
    expression -> PrettyPrint(out, lex_stream);
}

void AstBinaryExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    left_expression -> PrettyPrint(out, lex_stream);
    out << " ";
    Unicode::Out(out, lex_stream.Name(binary_operator_token));
    out << " ";
    right_expression -> PrettyPrint(out, lex_stream);
}

void AstConditionalExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    test_expression -> PrettyPrint(out, lex_stream);
    out << " ? ";
    true_expression -> PrettyPrint(out, lex_stream);
    out << " : ";
    false_expression -> PrettyPrint(out, lex_stream);
}

void AstAssignmentExpression::PrettyPrint(ostream& out, LexStream& lex_stream)
{
    left_hand_side -> PrettyPrint(out, lex_stream);
    out << " ";
    Unicode::Out(out, lex_stream.Name(assignment_operator_token));
    out << " ";
    expression -> PrettyPrint(out, lex_stream);
}
