
#include "ast.h"

static int num_locals;

// --------------------------------------------------------------------------

void Ast::PrintSemantics(ostream &tex, LexStream &lex_stream)
{
    cout << "Node number " << (int) kind << " does not contain a PrintSemantics routine\n";
}

// --------------------------------------------------------------------------

void AstBlock::PrintSemantics(ostream &tex, LexStream &lex_stream)
{
	for (int k = 0; k < NumStatements(); k++)
	{
		int kind = this -> Statement(k) -> kind;

		if (kind == BLOCK || kind == LOCAL_VARIABLE_DECLARATION)
			this -> Statement(k) -> PrintSemantics(tex, lex_stream);
	}
}

// --------------------------------------------------------------------------

void AstModifier::PrintSemantics(ostream &tex, LexStream &lex_stream)
{
	tex << "\\Modifier{";
	Unicode::Out(tex, lex_stream.Name(modifier_kind_token));
	tex << "}\n";
}

// --------------------------------------------------------------------------

void AstClassBody::PrintSemantics(ostream &tex, LexStream &lex_stream)
{
	int k, num_fields = 0, num_statics = 0, num_methods = 0;

	// fields
    for (k = 0; k < this -> NumClassBodyDeclarations(); k++)
	{
		int kind = this -> ClassBodyDeclaration(k) -> kind;
		if (kind == FIELD)
		{
			if (!num_fields)
				tex << "\\begin{Fields}\n";
			num_fields++;
			this -> ClassBodyDeclaration(k) -> PrintSemantics(tex, lex_stream);
		}
	}
	if (num_fields)
		tex << "\\end{Fields}\n";

	// static initializers
    for (k = 0; k < this -> NumClassBodyDeclarations(); k++)
	{
		int kind = this -> ClassBodyDeclaration(k) -> kind;
		if (kind == STATIC_INITIALIZER)
		{
			if (!num_statics)
			{
				tex << "\\begin{StaticInits}\n";
				Ast::verbatim = true;
			}
			this -> ClassBodyDeclaration(k) -> PrettyPrint(tex, lex_stream);
			num_statics++;
		}
	}
	if (num_statics)
	{
		Ast::verbatim = false;
		tex << "\\end{StaticInits}\n";
	}

	// methods
    for (int k = 0; k < this -> NumClassBodyDeclarations(); k++)
	{
		int kind = this -> ClassBodyDeclaration(k) -> kind;
		if (kind != FIELD && 
			kind != STATIC_INITIALIZER && 
			kind != EMPTY_DECLARATION)
		{
			if (!num_methods)
				tex << "\\begin{Methods}\n";
			tex << "\\Method{";
			if (this -> ClassBodyDeclaration(k) -> ConstructorDeclarationCast())
			{
				AstConstructorDeclaration *constructor = ClassBodyDeclaration(k) -> ConstructorDeclarationCast();
				AstMethodDeclarator *declarator = constructor -> constructor_declarator;

				for (int i = 0; i < constructor -> NumConstructorModifiers(); i++)
				{
					if (i)
						tex << " ";
					constructor -> ConstructorModifier(i) -> PrettyPrint(tex, lex_stream);
				}
				tex << "}{}{";
				Unicode::TeXOut(tex, lex_stream.Name(declarator -> identifier_token));
				tex << "}{";
				for (int j = 0; j < declarator -> NumFormalParameters(); j++)
				{
					if (j)
						tex << ", ";
					declarator -> FormalParameter(j) -> PrettyPrint(tex, lex_stream);
				}
			}
			else
			{
				AstMethodDeclaration *method = ClassBodyDeclaration(k) -> MethodDeclarationCast();
				AstMethodDeclarator *declarator = method -> method_declarator;

				for (int i = 0; i < method -> NumMethodModifiers(); i++)
				{
					if (i)
						tex << " ";
					method -> MethodModifier(i) -> PrettyPrint(tex, lex_stream);
				}
				tex << "}{";
				method -> type -> PrettyPrint(tex, lex_stream);
				tex << "}{";
				Unicode::TeXOut(tex, lex_stream.Name(declarator -> identifier_token));
				tex << "}{";
				for (int j = 0; j < declarator -> NumFormalParameters(); j++)
				{
					if (j)
						tex << ", ";
					declarator -> FormalParameter(j) -> PrettyPrint(tex, lex_stream);
				}
			}
			tex << "}\n";
			num_methods++;
		}
	}
	if (num_methods)
		tex << "\\end{Methods}\n";
}

// --------------------------------------------------------------------------

void AstClassDeclaration::PrintSemantics(ostream &tex, LexStream &lex_stream)
{
	tex << "\\begin{Modifiers}\n";
    for (int i = 0; i < this -> NumClassModifiers(); i++)
		this -> ClassModifier(i) -> PrintSemantics(tex, lex_stream);
	tex << "\\end{Modifiers}\n";

    if (super_opt)
	{
		tex << "\\begin{SuperClasses}\n\\SuperClass{";
		Unicode::TeXOut(tex, lex_stream.Name(super_opt -> SimpleNameCast() -> identifier_token));
		tex << "}\n\\end{SuperClasses}\n";
	}

    if (NumInterfaces() > 0)
    {
		tex << "\\begin{Interfaces}\n";
		for (int k = 0; k < NumInterfaces(); k++)
		{
			tex << "\\Interface{";
			this -> Interface(k) -> PrettyPrint(tex, lex_stream);
			tex << "}\n";
		}
		tex << "\\end{Interfaces}\n";
    }
}

// --------------------------------------------------------------------------

void AstFieldDeclaration::PrintSemantics(ostream &tex, LexStream &lex_stream)
{
	int num_modifiers = this -> NumVariableModifiers();

    for (int k = 0; k < this -> NumVariableDeclarators(); k++)
    {
		tex << "\\VariableDecl{";
		for (int i = 0; i < num_modifiers; i++)
		{
			if (i)
				tex << " ";
			Unicode::Out(tex, lex_stream.Name(this -> VariableModifier(i) -> modifier_kind_token));
		}
		tex << "}{";
		type -> PrettyPrint(tex, lex_stream);
		tex << "}{";
		this -> VariableDeclarator(k) -> PrettyPrint(tex, lex_stream);
		tex << "}\n";
    }
}

// --------------------------------------------------------------------------

void AstFormalParameter::PrintSemantics(ostream& tex, LexStream& lex_stream)
{
	tex << "\\Parameter{";
    for (int i = 0; i < this -> NumParameterModifiers(); i++)
    {
		if (i)
			tex << " ";
		Unicode::Out(tex, lex_stream.Name(this -> ParameterModifier(i) -> modifier_kind_token));
    }
	tex << "}{";
    type -> PrettyPrint(tex, lex_stream);
	tex << "}{";
    variable_declarator_name -> PrettyPrint(tex, lex_stream);
	tex << "}\n";
}

// --------------------------------------------------------------------------

void AstMethodDeclarator::PrintSemantics(ostream &tex, LexStream &lex_stream)
{
	if (this -> NumFormalParameters())
	{
		tex << "\\begin{Parameters}\n";
		for (int j = 0; j < this -> NumFormalParameters(); j++)
		{
			this -> FormalParameter(j) -> PrintSemantics(tex, lex_stream);
		}
		tex << "\\end{Parameters}\n";
	}
}

// --------------------------------------------------------------------------

void AstMethodDeclaration::PrintSemantics(ostream &tex, LexStream &lex_stream)
{
	if (this -> NumMethodModifiers())
	{
		tex << "\\begin{Modifiers}\n";
		for (int i = 0; i < this -> NumMethodModifiers(); i++)
			this -> MethodModifier(i) -> PrintSemantics(tex, lex_stream);
		tex << "\\end{Modifiers}\n";
	}

	tex << "\\ReturnType{";
	type -> PrettyPrint(tex, lex_stream);
	tex << "}\n";

    method_declarator -> PrintSemantics(tex, lex_stream);

	if (this -> NumThrows())
	{
		tex << "\\begin{Throws}\n";
		for (int k = 0; k < this -> NumThrows(); k++)
		{
			tex << "\\Throw{";
			Unicode::Out(tex, lex_stream.Name(Throw(k) -> SimpleNameCast() -> identifier_token));
			tex << "}\n";
		}
		tex << "\\end{Throws}\n";
	}

	num_locals = 0;
	if (method_body -> BlockCast())
		method_body -> PrintSemantics(tex, lex_stream);
	if (num_locals)
		tex << "\\end{Locals}\n";

	tex << "\\bigskip\n\n";
}

// --------------------------------------------------------------------------

void AstConstructorDeclaration::PrintSemantics(ostream &tex, LexStream &lex_stream)
{
	if (this -> NumConstructorModifiers())
	{
		tex << "\\begin{Modifiers}\n";
		for (int i = 0; i < this -> NumConstructorModifiers(); i++)
			this -> ConstructorModifier(i) -> PrintSemantics(tex, lex_stream);
		tex << "\\end{Modifiers}\n";
	}

    constructor_declarator -> PrintSemantics(tex, lex_stream);

	if (this -> NumThrows())
	{
		tex << "\\begin{Throws}\n";
		for (int k = 0; k < this -> NumThrows(); k++)
		{
			tex << "\\Throw{";
			Unicode::Out(tex, lex_stream.Name(Throw(k) -> SimpleNameCast() -> identifier_token));
			tex << "}\n";
		}
		tex << "\\end{Throws}\n";
	}

	num_locals = 0;
	if (constructor_body -> BlockCast())
		constructor_body -> PrintSemantics(tex, lex_stream);
	if (num_locals)
		tex << "\\end{Locals}\n";

	tex << "\\bigskip\n\n";
}

// --------------------------------------------------------------------------

void AstLocalVariableDeclarationStatement::PrintSemantics(ostream &tex, LexStream &lex_stream)
{
	if (!num_locals)
		tex << "\\begin{Locals}\n";

    for (int k = 0; k < this -> NumVariableDeclarators(); k++)
    {
		tex << "\\Local{";
		type -> PrettyPrint(tex, lex_stream);
		tex << "}{";
		this -> VariableDeclarator(k) -> variable_declarator_name -> PrettyPrint(tex, lex_stream);
		tex << "}\n";
		num_locals++;
    }
}
